@@ -3,10 +3,16 @@ import sys
 import yaml
 import getpass
 
+from twisted.internet import defer, reactor
+from twisted.internet.endpoints import TCP4ClientEndpoint, connectProtocol
+
 from os.path import abspath, expanduser
 
-from ooni import otime, geoip
-from ooni.utils import Storage
+from ooni.utils.net import ConnectAndCloseProtocol
+from ooni import geoip
+from ooni.utils import Storage, log
+from ooni import errors
+
 
 class OConfig(object):
     _custom_home = None
@@ -19,8 +25,6 @@ class OConfig(object):
         self.tor_state = None
         # This is used to store the probes IP address obtained via Tor
         self.probe_ip = geoip.ProbeIP()
-        # This is used to keep track of the state of the sniffer
-        self.sniffer_running = None
         self.logging = True
         self.basic = Storage()
         self.advanced = Storage()
@@ -49,6 +53,7 @@ class OConfig(object):
         self.inputs_directory = os.path.join(self.ooni_home, 'inputs')
         self.decks_directory = os.path.join(self.ooni_home, 'decks')
         self.reports_directory = os.path.join(self.ooni_home, 'reports')
+        self.report_log_file = os.path.join(self.ooni_home, 'reporting.yml')
 
         if self.global_options.get('configfile'):
             config_file = self.global_options['configfile']
@@ -91,10 +96,8 @@ class OConfig(object):
                     else:
                         w.write(line)
 
-    def read_config_file(self):
-        try:
-            with open(self.config_file) as f: pass
-        except IOError:
+    def read_config_file(self, check_incoherences=False):
+        if not os.path.isfile(self.config_file):
             print "Configuration file does not exist."
             self._create_config_file()
             self.read_config_file()
@@ -103,17 +106,61 @@ class OConfig(object):
             config_file_contents = '\n'.join(f.readlines())
             configuration = yaml.safe_load(config_file_contents)
 
-            for setting in ['basic', 'reports', 'advanced', 'privacy', 'tor']:
-                try:
-                    for k, v in configuration[setting].items():
-                        getattr(self, setting)[k] = v
-                except AttributeError:
-                    pass
+        for setting in configuration.keys():
+            if setting in dir(self) and configuration[setting] is not None:
+                for k, v in configuration[setting].items():
+                    getattr(self, setting)[k] = v
         self.set_paths()
 
-    def generate_pcap_filename(self, testDetails):
-        test_name, start_time = testDetails['test_name'], testDetails['start_time']
-        start_time = otime.epochToTimestamp(start_time)
-        return "report-%s-%s.%s" % (test_name, start_time, "pcap")
+        if check_incoherences:
+            self.check_incoherences(configuration)
+
+    def check_incoherences(self, configuration):
+        incoherent = []
+
+        if configuration['advanced']['interface'] != 'auto':
+            from scapy.all import get_if_list
+            if configuration['advanced']['interface'] not in get_if_list():
+                incoherent.append('advanced:interface')
+
+        self.log_incoherences(incoherent)
+
+    def log_incoherences(self, incoherences):
+        if len(incoherences) > 0:
+            if len(incoherences) > 1:
+                incoherent_pretty = ", ".join(incoherences[:-1]) + ' and ' + incoherences[-1]
+            else:
+                incoherent_pretty = incoherences[0]
+            log.err("You must set properly %s in %s." % (incoherent_pretty, self.config_file))
+            raise errors.ConfigFileIncoherent
+
+    @defer.inlineCallbacks
+    def check_tor(self):
+        """
+        Called only when we must start tor by director.start
+        """
+        incoherent = []
+        if not self.advanced.start_tor:
+            if self.tor.socks_port is None:
+                incoherent.append('tor:socks_port')
+            else:
+                socks_port_ep = TCP4ClientEndpoint(reactor,
+                                                   "localhost",
+                                                   self.tor.socks_port)
+                try:
+                    yield connectProtocol(socks_port_ep, ConnectAndCloseProtocol())
+                except Exception:
+                    incoherent.append('tor:socks_port')
+
+            if self.tor.control_port is not None:
+                control_port_ep = TCP4ClientEndpoint(reactor,
+                                                     "localhost",
+                                                     self.tor.control_port)
+                try:
+                    yield connectProtocol(control_port_ep, ConnectAndCloseProtocol())
+                except Exception:
+                    incoherent.append('tor:control_port')
+
+            self.log_incoherences(incoherent)
 
 config = OConfig()
