@@ -3,11 +3,14 @@ import sys
 import yaml
 
 from twisted.internet import defer
-from twisted.trial import unittest
 
 from ooni.tests import is_internet_connected
+from ooni.tests.bases import ConfigTestCase
 from ooni.settings import config
 from ooni.oonicli import runWithDirector
+from ooni.utils import checkForRoot
+from ooni.errors import InsufficientPrivileges
+
 
 def verify_header(header):
     assert 'input_hashes' in header.keys()
@@ -20,37 +23,74 @@ def verify_header(header):
     assert 'test_name' in header.keys()
     assert 'test_version' in header.keys()
 
+
 def verify_entry(entry):
     assert 'input' in entry
 
+config_includepcap = """
+basic:
+    logfile: ~/.ooni/ooniprobe.log
+privacy:
+    includeip: false
+    includeasn: true
+    includecountry: true
+    includecity: false
+    includepcap: true
+reports:
+    pcap: null
+    collector: null
+advanced:
+    geoip_data_dir: /usr/share/GeoIP
+    debug: false
+    interface: auto
+    start_tor: false
+    measurement_timeout: 60
+    measurement_retries: 2
+    measurement_concurrency: 10
+    reporting_timeout: 80
+    reporting_retries: 3
+    reporting_concurrency: 15
+    data_dir: /usr/share/ooni
+    oonid_api_port: 8042
+tor:
+    socks_port: 9050
 
-class TestRunDirector(unittest.TestCase):
+"""
+
+
+class TestRunDirector(ConfigTestCase):
     def setUp(self):
         if not is_internet_connected():
             self.skipTest("You must be connected to the internet to run this test")
+        try:
+            checkForRoot()
+        except InsufficientPrivileges:
+            self.skipTest("You must be root to run this test")
         config.tor.socks_port = 9050
         config.tor.control_port = None
+        self.filenames = ['example-input.txt']
         with open('example-input.txt', 'w+') as f:
             f.write('http://torproject.org/\n')
             f.write('http://bridges.torproject.org/\n')
             f.write('http://blog.torproject.org/\n')
 
     def tearDown(self):
-        try:
-            os.remove('test_report.yaml')
-        except:
-            pass
-        try:
-            os.remove('example-input.txt')
-        except:
-            pass
+        super(TestRunDirector, self).tearDown()
+        for filename in self.filenames:
+            if os.path.exists(filename):
+                os.remove(filename)
+        self.filenames = []
 
     @defer.inlineCallbacks
-    def run_test(self, test_name, args, verify_function):
-        output_file = 'test_report.yaml'
-        sys.argv = ['', '-n', '-o', output_file, test_name]
-        sys.argv.extend(args)
-        yield runWithDirector(False, False)
+    def run_helper(self, test_name, nettest_args, verify_function, ooni_args=()):
+        output_file = os.path.abspath('test_report.yamloo')
+        self.filenames.append(output_file)
+        oldargv = sys.argv
+        sys.argv = ['']
+        sys.argv.extend(ooni_args)
+        sys.argv.extend(['-n', '-o', output_file, test_name])
+        sys.argv.extend(nettest_args)
+        yield runWithDirector(False, False, False)
         with open(output_file) as f:
             entries = yaml.safe_load_all(f)
             header = entries.next()
@@ -61,6 +101,7 @@ class TestRunDirector(unittest.TestCase):
         verify_header(header)
         verify_entry(first_entry)
         verify_function(first_entry)
+        sys.argv = oldargv
 
     @defer.inlineCallbacks
     def test_http_requests(self):
@@ -72,9 +113,10 @@ class TestRunDirector(unittest.TestCase):
             assert 'factor' in entry
             assert 'headers_diff' in entry
             assert 'headers_match' in entry
-        yield self.run_test('blocking/http_requests',
-                      ['-u', 'http://torproject.org/'],
-                      verify_function)
+
+        yield self.run_helper('blocking/http_requests',
+                              ['-u', 'http://torproject.org/'],
+                              verify_function)
 
     @defer.inlineCallbacks
     def test_http_requests_with_file(self):
@@ -86,9 +128,10 @@ class TestRunDirector(unittest.TestCase):
             assert 'factor' in entry
             assert 'headers_diff' in entry
             assert 'headers_match' in entry
-        yield self.run_test('blocking/http_requests',
-                      ['-f', 'example-input.txt'],
-                      verify_function)
+
+        yield self.run_helper('blocking/http_requests',
+                              ['-f', 'example-input.txt'],
+                              verify_function)
 
     @defer.inlineCallbacks
     def test_dnsconsistency(self):
@@ -97,11 +140,12 @@ class TestRunDirector(unittest.TestCase):
             assert 'control_resolver' in entry
             assert 'tampering' in entry
             assert len(entry['tampering']) == 1
-        yield self.run_test('blocking/dns_consistency',
-                            ['-b', '8.8.8.8:53',
-                             '-t', '8.8.8.8',
-                             '-f', 'example-input.txt'],
-                            verify_function)
+
+        yield self.run_helper('blocking/dns_consistency',
+                              ['-b', '8.8.8.8:53',
+                               '-t', '8.8.8.8',
+                               '-f', 'example-input.txt'],
+                              verify_function)
 
     @defer.inlineCallbacks
     def test_http_header_field_manipulation(self):
@@ -118,6 +162,23 @@ class TestRunDirector(unittest.TestCase):
             assert 'request_line_capitalization' in entry['tampering']
             assert 'total' in entry['tampering']
 
-        yield self.run_test('manipulation/http_header_field_manipulation',
-                            ['-b', 'http://64.9.225.221'],
-                           verify_function)
+        yield self.run_helper('manipulation/http_header_field_manipulation',
+                              ['-b', 'http://64.9.225.221'],
+                              verify_function)
+
+    @defer.inlineCallbacks
+    def test_sniffing_activated(self):
+        filename = os.path.abspath('test_report.pcap')
+        self.filenames.append(filename)
+        conf_file = os.path.abspath('fake_config.conf')
+        with open(conf_file, 'w') as cfg:
+            cfg.writelines(config_includepcap)
+        self.filenames.append(conf_file)
+
+        def verify_function(_):
+            assert os.path.exists(filename)
+            self.assertGreater(os.stat(filename).st_size, 0)
+        yield self.run_helper('blocking/http_requests',
+                              ['-f', 'example-input.txt'],
+                              verify_function, ooni_args=['-f', conf_file])
+        config.scapyFactory.connectionLost('')
