@@ -1,13 +1,12 @@
-import logging
 import string
 import random
 import glob
-import yaml
-import imp
 import os
 
+from ooni import otime
 from ooni import errors
 
+
 class Storage(dict):
     """
     A Storage object is like a dictionary except `obj.foo` can be used
@@ -25,10 +24,11 @@ class Storage(dict):
         >>> o.a
         None
     """
+
     def __getattr__(self, key):
         try:
             return self[key]
-        except KeyError, k:
+        except KeyError:
             return None
 
     def __setattr__(self, key, value):
@@ -50,10 +50,12 @@ class Storage(dict):
         for (k, v) in value.items():
             self[k] = v
 
+
 def checkForRoot():
     if os.getuid() != 0:
         raise errors.InsufficientPrivileges
 
+
 def randomSTR(length, num=True):
     """
     Returns a random all uppercase alfa-numerical (if num True) string long length
@@ -63,6 +65,7 @@ def randomSTR(length, num=True):
         chars += string.digits
     return ''.join(random.choice(chars) for x in range(length))
 
+
 def randomstr(length, num=True):
     """
     Returns a random all lowercase alfa-numerical (if num True) string long length
@@ -72,6 +75,7 @@ def randomstr(length, num=True):
         chars += string.digits
     return ''.join(random.choice(chars) for x in range(length))
 
+
 def randomStr(length, num=True):
     """
     Returns a random a mixed lowercase, uppercase, alfanumerical (if num True)
@@ -82,6 +86,7 @@ def randomStr(length, num=True):
         chars += string.digits
     return ''.join(random.choice(chars) for x in range(length))
 
+
 def pushFilenameStack(filename):
     """
     Takes as input a target filename and checks to see if a file by such name
@@ -93,7 +98,7 @@ def pushFilenameStack(filename):
     Args:
         filename (str): the path to filename that you wish to create.
     """
-    stack = glob.glob(filename+".*")
+    stack = glob.glob(filename + ".*")
     stack.sort(key=lambda x: int(x.split('.')[-1]))
     for f in reversed(stack):
         c_idx = f.split(".")[-1]
@@ -101,4 +106,39 @@ def pushFilenameStack(filename):
         new_idx = int(c_idx) + 1
         new_filename = "%s.%s" % (c_filename, new_idx)
         os.rename(f, new_filename)
-    os.rename(filename, filename+".1")
+    os.rename(filename, filename + ".1")
+
+
+def generate_filename(testDetails, prefix=None, extension=None, filename=None):
+    """
+    Returns a filename for every test execution.
+
+    It's used to assure that all files of a certain test have a common basename but different
+    extension.
+    """
+    if filename is None:
+        test_name, start_time = testDetails['test_name'], testDetails['start_time']
+        start_time = otime.epochToTimestamp(start_time)
+        suffix = "%s-%s" % (test_name, start_time)
+        basename = '%s-%s' % (prefix, suffix) if prefix is not None else suffix
+        final_filename = '%s.%s' % (basename, extension) if extension is not None else basename
+    else:
+        if extension is not None:
+            basename = filename.split('.')[0] if '.' in filename else filename
+            final_filename = '%s.%s' % (basename, extension)
+        else:
+            final_filename = filename
+
+    return final_filename
+
+def sanitize_options(options):
+    """
+    Strips all possible user identifying information from the ooniprobe test
+    options.
+    Currently only strips leading directories from filepaths.
+    """
+    sanitized_options = []
+    for option in options:
+        option = os.path.basename(option)
+        sanitized_options.append(option)
+    return sanitized_options
